'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _react = require('react');

var getNodeType = function getNodeType(node) {
  if (typeof node.type === 'function') {
    if (node.type.prototype.isReactComponent || node.type.prototype.isPureReactComponent) {
      return 'statefull';
    }
    return 'stateless';
  }

  if (!node.type && node) {
    return node.constructor.name.toLocaleLowerCase();
  }

  return node.type.constructor.name.toLocaleLowerCase();
};

var initComponent = function initComponent(node, context) {
  switch (getNodeType(node)) {
    case 'statefull':
      {
        // eslint-disable-next-line new-cap
        var element = new node.type(node.props, context);

        if (element.componentWillMount) {
          try {
            element.componentWillMount();
          } catch (e) {
            // eslint-disable-next-line no-console
            console.log('error');
          }
        }

        var component = void 0;

        try {
          component = element.render();
        } catch (e) {
          component = [];
        }

        return [component, Object.assign(context, element.getChildContext ? element.getChildContext() : {})];
      }
    case 'stateless':
      return [node.type(node.props, context), context];
    default:
      return [node, context];
  }
};

var findPrefetches = function findPrefetches(node) {
  var context = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var prefetchMethod = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'prefetch';

  var prefetches = [];

  if (!node) {
    return prefetches;
  }

  if (node.type && node.type[prefetchMethod] && !node.type.WrappedComponent && !node.type.Naked) {
    prefetches.push([node.type[prefetchMethod], node.props]);
  }

  var _initComponent = initComponent(node, context),
      _initComponent2 = _slicedToArray(_initComponent, 2),
      component = _initComponent2[0],
      childContext = _initComponent2[1];

  if (component) {
    var nodeType = getNodeType(component);

    if (nodeType === 'stateless' || nodeType === 'statefull') {
      prefetches = prefetches.concat(findPrefetches(component, context, prefetchMethod));
    } else {
      var children = void 0;

      if (nodeType === 'array') {
        children = component;
      } else if (component.props && component.props.children) {
        // eslint-disable-next-line prefer-destructuring
        children = component.props.children;
      }

      if (children) {
        _react.Children.forEach(children, function (child) {
          var childPrefetches = findPrefetches(child, childContext, prefetchMethod);

          if (childPrefetches.length) {
            prefetches = prefetches.concat(childPrefetches);
          }
        });
      }
    }
  }

  return prefetches;
};

exports.default = findPrefetches;